package application;
import java.sql.*;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;


public class DBManager {
	
	//Verbindung mit Datenbank herstellen
	public Connection getConnection() throws ClassNotFoundException, SQLException {// ClassnotFound wenn Tippfehler oder Treiber nicht eingebunden
		
		Connection con = null;	
		
		Class.forName("com.mysql.cj.jdbc.Driver");		// installiert den Driver für die anwendung. Regestriert die Datenbank "Hallo ich bin jetzt da"
		con = DriverManager.getConnection(  // baut die Verbindung zur Datenbank auf
					"jdbc:mysql://localhost:3305/Firma?serverTimezone=UTC", //jdbc immer wenn man auf eine Datenbank zugreift //durch 3306 (Durchwahl) lande ich beim MySQL Server //localhost = mein Rechner // Schule1 = Datenbank
					"root",                                 // User
					"root"                                  // Passwort
					);
		return con;
	}
	
	 public boolean saveOrUpdateKunde(String vorname, String nachname, String email,
	            String gebDat, String preis, String paketDatum, Connection con) {
	        String sqlSelect = "SELECT * FROM Kunde WHERE Vorname = ? ";
	        String sqlInsert = "INSERT INTO Kunde (Vorname, Email, GebDat, Preis, PaketDatum) VALUES (?, ?, ?, ?, ?)";
	        String sqlUpdate = "UPDATE Kunde SET Email = ?, GebDat = ?, Preis = ?, PaketDatum = ? WHERE Vorname = ? ";

	        try {
	            // Prüfen, ob der Kunde bereits existiert
	            PreparedStatement stmtSelect = con.prepareStatement(sqlSelect);
	            stmtSelect.setString(1, vorname);
	            ResultSet rs = stmtSelect.executeQuery();

	            if (rs.next()) {
	                // Kunde existiert bereits, also aktualisieren wir die Daten
	                PreparedStatement stmtUpdate = con.prepareStatement(sqlUpdate);
	                stmtUpdate.setString(1, email);
	                stmtUpdate.setString(2, gebDat);
	                stmtUpdate.setString(3, preis);

	                // Überprüfen und konvertieren des Paketdatums
	                LocalDate paketLocalDate = convertToDate(paketDatum);
	                if (paketLocalDate != null) {
	                    stmtUpdate.setDate(4, java.sql.Date.valueOf(paketLocalDate));
	                } else {
	                    stmtUpdate.setNull(4, Types.DATE);
	                }

	                stmtUpdate.setString(5, vorname);
	                stmtUpdate.executeUpdate();
	                stmtUpdate.close();
	                return true; // Aktualisierung erfolgreich
	            } else {
	                // Kunde existiert nicht, also fügen wir ihn hinzu
	                PreparedStatement stmtInsert = con.prepareStatement(sqlInsert);
	                stmtInsert.setString(1, vorname);
	                stmtInsert.setString(2, email);
	                stmtInsert.setString(3, gebDat);
	                stmtInsert.setString(4, preis);

	                // Überprüfen und konvertieren des Paketdatums
	                LocalDate paketLocalDate = convertToDate(paketDatum);
	                if (paketLocalDate != null) {
	                    stmtInsert.setDate(5, java.sql.Date.valueOf(paketLocalDate));
	                } else {
	                    stmtInsert.setNull(5, Types.DATE);
	                }

	                stmtInsert.executeUpdate();
	                stmtInsert.close();
	                return true; // Hinzufügung erfolgreich
	            }
	        } catch (SQLException e) {
	            e.printStackTrace();
	            return false; // Ein Fehler ist aufgetreten
	        }
	    }
	 
	 
	 /*public String convertDateToSQLFormat(String dateString) throws DateTimeParseException {
	        DateTimeFormatter inputFormat = DateTimeFormatter.ofPattern("dd-MM-yyyy");
	        DateTimeFormatter outputFormat = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	        LocalDate date = LocalDate.parse(dateString, inputFormat);
	        return date.format(outputFormat);
	    }*/

	    // Methode zur Konvertierung des Datumsformats
	    private LocalDate convertToDate(String date) {
	        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd-MM-yyyy");
	        try {
	            return LocalDate.parse(date, formatter);
	        } catch (DateTimeParseException e) {
	            return null; // Ungültiges Datum
	        }
	    }

	// Methode zur Überprüfung des Datumsformats
	private boolean isValidDateFormat(String date) {
	    // Hier kannst du deine eigene Logik zur Überprüfung des Datumsformats implementieren
	    // Beispiel: Überprüfen, ob das Datum im Format "yyyy-MM-dd" ist
	    return date.matches("\\d{4}-\\d{2}-\\d{2}");
	}

	
	public boolean dropKunde(String vorname, Connection con) {
	    String sql = "DELETE FROM Kunde WHERE Vorname = ? ";
	    try {
	        PreparedStatement stmt = con.prepareStatement(sql);
	        stmt.setString(1, vorname);

	        int rowsAffected = stmt.executeUpdate();
	        stmt.close();
	        return rowsAffected > 0; // Gibt true zurück, wenn ein Kunde gelöscht wurde
	    } catch (SQLException e) {
	        e.printStackTrace();
	        return false; // Fehler aufgetreten
	    }
	}
	
	public void releaseConnection (Connection con) throws SQLException { // schließt die Connection
		
		if (con != null)
			con.close();	
	}
	//fFunktion für getPakte (hätte man auch wie bei anderen Funktionen in der Main definieren können aber egal
	public ResultSet getPakete(Connection con) throws SQLException {
        String query = "SELECT * FROM Kunde WHERE PaketDatum < DATE_SUB(CURDATE(), INTERVAL 1 YEAR)"; //Derzeit: Kunde wird angezeigt, wenn letzte Plakette ein Jahr her ist
        Statement stmt = con.createStatement();
        return stmt.executeQuery(query);
    }
	
	//wird benötigt falls Kunde schon angelegt ist aber trz. verändert werden soll ( onLogInClick() )
	public void insertPersonInfo(Kunde a, Connection con) throws SQLException {
	    String sql = "INSERT INTO Kunde (Vorname, Email, GebDat, Preis, PaketDatum) VALUES (?, ?, ?, ?, ?)";
	    PreparedStatement stm = null;
	    try {
	        stm = con.prepareStatement(sql);
	        stm.setString(1, a.getVorname());

	        stm.setString(3, a.getEmail());
	        stm.setString(4, a.getGebDat());
	        stm.setString(5, a.getPreis());
	        
	        // PaketDatum nur setzen, wenn es nicht null ist
	        if (a.getPakette() != null) {
	            stm.setDate(6, java.sql.Date.valueOf(a.getPakette()));
	        } else {
	            stm.setNull(6, Types.DATE); // Null setzen, wenn kein Datum vorhanden ist
	        }

	        stm.executeUpdate();
	    } finally {
	        if (stm != null)
	            stm.close();
	    }
	}
	
	
	//Kunden anwählen (in der Datenbank)
	public ResultSet getKunde(Kunde b, Connection con) throws SQLException {
	    // Normalisiere die Eingabewerte
	    String vorname = b.getVorname().trim().toLowerCase();

	    // SQL-Statement mit LIKE-Operator damit Abstände und Groß-Kleinschreibung egal ist
	    String sql = "SELECT * FROM Kunde WHERE TRIM(LOWER(Vorname)) LIKE ?";
	    PreparedStatement stm = null;
	    ResultSet rs = null;

	    try {
	        stm = con.prepareStatement(sql); // Prepare the statement
	        
	        // Setze den Parameter in der SQL-Anweisung
	        stm.setString(1, "%" + vorname + "%"); // '%' für LIKE, Index 1
	        
	        rs = stm.executeQuery(); // Execute the query
	    } catch (SQLException e) {
	        e.printStackTrace(); // Log the exception
	        throw e; // Rethrow if needed
	    }

	    return rs; // Return the ResultSet (it is the caller's responsibility to close it)
	}



	
}