package application;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;
import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.control.TextField;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;


public class MavenApplication extends Application {
    private ListView<String> vornameListView;

    // Grafische Elemente
    TextField VNameTextFeld;
    TextField NNameTextFeld;
    TextField EmailTextFeld;
    TextField GebDatTextFeld;
    TextField PreisTextFeld;
    TextField PaketteTextFeld;
    Text actiontarget;



    // ...



    // ...

    @Override
    public void start(Stage primaryStage) {
        primaryStage.setTitle("Kundenverwaltung");

        // Initialisierung der Eingabefelder und Schaltflächen
        VNameTextFeld = new TextField();
        NNameTextFeld = new TextField(); // Nachname TextField
        EmailTextFeld = new TextField(); // E-Mail TextField
        GebDatTextFeld = new TextField(); // Geburtsdatum TextField
        PreisTextFeld = new TextField(); // Preis TextField
        PaketteTextFeld = new TextField(); // Paketdatum TextField

        Button loginButton = new Button("Daten speichern");
        Button holenButton = new Button("Kunden holen");
        Button alterButton = new Button("Durchschnittsalter");
        Button anzeigenButton = new Button("Plaketten anzeigen");
        Button dropButton = new Button("Drop Kunde");
        actiontarget = new Text();

        // Setzen Sie die gleiche Breite für alle Buttons
        loginButton.setPrefWidth(150);
        holenButton.setPrefWidth(150);
        alterButton.setPrefWidth(150);
        anzeigenButton.setPrefWidth(150);
        dropButton.setPrefWidth(150);

        // Scrollable text area
        TextArea outputTextArea = new TextArea();
        outputTextArea.setEditable(false);
        outputTextArea.setPrefWidth(200);
        outputTextArea.setPrefHeight(600);

        ScrollPane scrollPane = new ScrollPane(outputTextArea);
        scrollPane.setFitToWidth(true);
        scrollPane.setFitToHeight(true);

        // Layout für die Textfelder und Buttons
        VBox inputBox = new VBox(10);
        inputBox.setPadding(new Insets(20));
        inputBox.setAlignment(Pos.CENTER);
        inputBox.getChildren().addAll(
                new Label("Name (Vorname Nachname):"), VNameTextFeld,
                new Label("PlaketteDatum TT-MM-JJJJ:"), PaketteTextFeld, // Move Paketdatum under Vorname
                new Label("E-Mail:"), EmailTextFeld,
                new Label("Geburtsjahr JJJJ:"), GebDatTextFeld,
                new Label("Preis:"), PreisTextFeld,
                loginButton, holenButton, alterButton, anzeigenButton, dropButton, actiontarget
        );

        // Aktionen für die Schaltflächen
        loginButton.setOnAction(e -> onLoginClick());
        holenButton.setOnAction(e -> onHolenClick());
        alterButton.setOnAction(e -> onAlterClick());
        anzeigenButton.setOnAction(e -> onPaketeAnzeigenClick());
        dropButton.setOnAction(e -> onDropClick());

        // Echtzeitsuche
        vornameListView = new ListView<>();
        ObservableList<String> vornameList = FXCollections.observableArrayList();

        // Suchfeld für Vornamen
        VNameTextFeld.textProperty().addListener((observable, oldValue, newValue) -> {
            String query = newValue.toLowerCase();
            vornameList.clear();
            try {
                Connection con = new DBManager().getConnection();
                String sql = "SELECT Vorname FROM Kunde WHERE Vorname LIKE ?";
                PreparedStatement stmt = con.prepareStatement(sql);
                stmt.setString(1, query + "%");
                ResultSet rs = stmt.executeQuery();
                while (rs.next()) {
                    vornameList.add(rs.getString("Vorname"));
                }
                vornameListView.setItems(vornameList);
                stmt.close();
                rs.close();
                con.close();
            } catch (SQLException | ClassNotFoundException e) {
                e.printStackTrace();
            }
        });

        // Listener für Vorname ListView
        vornameListView.setOnMouseClicked(event -> {
            String selectedVorname = vornameListView.getSelectionModel().getSelectedItem();
            if (selectedVorname != null) {
                VNameTextFeld.setText(selectedVorname);
                vornameListView.getItems().clear(); // Liste nach Auswahl leeren
            }
        });

        // Layout für die ListView
        VBox searchBox = new VBox(10);
        searchBox.setPadding(new Insets(20));
        searchBox.getChildren().addAll(new Label("Suchergebnisse:"), vornameListView);

        // Hauptlayout
        HBox mainLayout = new HBox(10, inputBox, searchBox); // Eingabefeld und Suchfeld in HBox hinzufügen
        mainLayout.setAlignment(Pos.TOP_CENTER); // Zentriert das Layout

        Scene scene = new Scene(mainLayout, 800, 600); // Größe der Szene anpassen
        primaryStage.setScene(scene);
        primaryStage.show();
    }



    private void onAlterClick() {
        DBManager dbManager = new DBManager();
        Connection con = null;
        ResultSet rs = null;

        try {
            con = dbManager.getConnection();
            String sql = "SELECT AVG(YEAR(NOW()) - CAST(GebDat AS UNSIGNED)) AS Durchschnittsalter FROM Kunde WHERE GebDat REGEXP '^[0-9]{4}$';";
            PreparedStatement stm = con.prepareStatement(sql);
            rs = stm.executeQuery();

            if (rs.next()) {
                double durchschnittsalter = rs.getDouble("Durchschnittsalter");
                actiontarget.setText("Durchschnittsalter der Kunden: " + String.format("%.2f", durchschnittsalter) + " Jahre");
            } else {
                actiontarget.setText("Keine Daten vorhanden.");
            }

        } catch (SQLException | ClassNotFoundException e) {
            e.printStackTrace();
            actiontarget.setText("Ein Fehler ist aufgetreten: " + e.getMessage());
        } finally {
            // Ressourcen schließen
            try {
                if (rs != null) {
                    rs.close();
                }
                if (con != null) {
                    dbManager.releaseConnection(con);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private void onPaketeAnzeigenClick() {
        DBManager dbManager = new DBManager();
        Connection con = null;
        ResultSet rs = null;

        try {
            con = dbManager.getConnection();
            rs = dbManager.getPakete(con);

            StringBuilder sb = new StringBuilder();

            if (rs != null) {
                while (rs.next()) {
                    String vorname = rs.getString("Vorname");
                    String email = rs.getString("Email");
                    LocalDate paketDatum = null;

                    if (rs.getDate("PaketDatum") != null) {
                        paketDatum = rs.getDate("PaketDatum").toLocalDate();
                    }

                    String paketDatumStr = (paketDatum != null) ? paketDatum.toString() : "Kein PlaketteDatum vorhanden";

                    sb.append(" ").append(vorname)
                    		.append(", ").append(paketDatumStr)
                            .append(", Email: ").append(email).append("\n");
                }

                // Create a new stage for the results window
                Stage resultsStage = new Stage();
                resultsStage.setTitle("Plakette Anzeigen");

                // Create a text area to display the results
                TextArea resultsTextArea = new TextArea();
                resultsTextArea.setEditable(false);
                resultsTextArea.setPrefWidth(200);
                resultsTextArea.setPrefHeight(600);
                resultsTextArea.setText(sb.toString());

                // Create a scroll pane for the text area
                ScrollPane scrollPane = new ScrollPane(resultsTextArea);
                scrollPane.setFitToWidth(true);
                scrollPane.setFitToHeight(true);
                scrollPane.setVbarPolicy(ScrollPane.ScrollBarPolicy.ALWAYS); // Make the scroll pane vertically scrollable

                // Create a scene for the results window
                Scene resultsScene = new Scene(scrollPane, 300, 400);

                // Set the scene for the results stage and show it
                resultsStage.setScene(resultsScene);
                resultsStage.show();
            } else {
                // Display an error message in the results window's text area
                Stage resultsStage = new Stage();
                resultsStage.setTitle("Plakette Anzeigen");

                TextArea resultsTextArea = new TextArea();
                resultsTextArea.setEditable(false);
                resultsTextArea.setPrefWidth(200);
                resultsTextArea.setPrefHeight(600);
                resultsTextArea.setText("Keine Plakette gefunden.");

                ScrollPane scrollPane = new ScrollPane(resultsTextArea);
                scrollPane.setFitToWidth(true);
                scrollPane.setFitToHeight(true);
                scrollPane.setVbarPolicy(ScrollPane.ScrollBarPolicy.ALWAYS); // Make the scroll pane vertically scrollable

                Scene resultsScene = new Scene(scrollPane, 300, 400);

                resultsStage.setScene(resultsScene);
                resultsStage.show();
            }
        } catch (SQLException | ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (con != null) {
                    dbManager.releaseConnection(con);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }





    private void onHolenClick() {
        DBManager dbManager = new DBManager();
        Connection con = null;
        ResultSet rs = null;

        String vorname = VNameTextFeld.getText();
        String nachname = NNameTextFeld.getText(); // Nachname-Feld ist jetzt auch vorhanden

        Kunde kunde = new Kunde(vorname, nachname, "", "", "", LocalDate.now());

        try {
            con = dbManager.getConnection();
            rs = dbManager.getKunde(kunde, con);

            if (rs != null && rs.next()) {
                String vornameDb = rs.getString("Vorname");
                String email = rs.getString("Email");
                String gebDat = rs.getString("GebDat");
                String preis = rs.getString("Preis");
                LocalDate paketDatum = null;

                if (rs.getDate("PaketDatum") != null) {
                    paketDatum = rs.getDate("PaketDatum").toLocalDate();
                }

                String paketDatumStr = (paketDatum != null) ? paketDatum.toString() : "Kein Paketdatum vorhanden";

                actiontarget.setText("Vorname: " + vornameDb + ", E-Mail: " + email +
                        ", Geburtstag: " + gebDat + ", Preis: " + preis + ", PlaketteDatum: " + paketDatumStr);
            } else {
                actiontarget.setText("Kunde nicht gefunden.");
            }
        } catch (SQLException | ClassNotFoundException e) {
            e.printStackTrace();
            actiontarget.setText("Ein Fehler ist aufgetreten: " + e.getMessage());
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (con != null) {
                    dbManager.releaseConnection(con);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private void onLoginClick() {
        DBManager dbManager = new DBManager();
        Connection con = null;

        String vorname = VNameTextFeld.getText();
        String nachname = NNameTextFeld.getText(); // Füge nachname hinzu
        String email = EmailTextFeld.getText();
        String gebDat = GebDatTextFeld.getText();
        String preis = PreisTextFeld.getText();
        String paketDatum = PaketteTextFeld.getText();

        // Vorname muss ausgefüllt sein
        if (vorname.isEmpty()) {
            actiontarget.setText("Vorname ist erforderlich.");
            return;
        }

        // Null-Check für NNameTextFeld
        if (NNameTextFeld == null) {
            System.out.println("NNameTextFeld is null");
            return;
        }

        try {
            con = dbManager.getConnection();
            // Übergebe auch nachname
            boolean success = dbManager.saveOrUpdateKunde(vorname, nachname, email, gebDat, preis, paketDatum, con);
            if (success) {
                actiontarget.setText("Kunde erfolgreich gespeichert/aktualisiert.");
            } else {
                actiontarget.setText("Kunde konnte nicht gespeichert/aktualisiert werden.");
            }
        } catch (SQLException | ClassNotFoundException e) {
            e.printStackTrace();
            actiontarget.setText("Ein Fehler ist aufgetreten: " + e.getMessage());
        } finally {
            try {
                if (con != null) {
                    dbManager.releaseConnection(con);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    
    

    private void onDropClick() {
        DBManager dbManager = new DBManager();
        Connection con = null;
        String vorname = VNameTextFeld.getText();
 

        try {
            con = dbManager.getConnection();
            boolean success = dbManager.dropKunde(vorname, con);
            if (success) {
                actiontarget.setText("Kunde erfolgreich gelöscht.");
            } else {
                actiontarget.setText("Kunde konnte nicht gelöscht werden.");
            }
        } catch (SQLException | ClassNotFoundException e) {
            e.printStackTrace();
            actiontarget.setText("Ein Fehler ist aufgetreten: " + e.getMessage());
        } finally {
            try {
                if (con != null) {
                    dbManager.releaseConnection(con);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    public static void main(String[] args) {
        launch(args);
    }
}
